@@ -30,8 +30,11 @@
 ##'   supplied, a suitable label will be generated.
 ##' @param ylab character or expression; the label for the y axis. If not
 ##'   supplied, a suitable label will be generated.
-##' @param alpha numeric; the level of alpha transparency for the reference
+##' @param ci_col,ci_alpha fill colour and alpha transparency for the reference
 ##'   interval when `method = "simulate"`.
+##' @param point_col,point_alpha colour and alpha transparency for points on the
+##'   QQ plot.
+##' @param line_col colour used to draw the 1:1 reference line.
 ##'
 ##' @inheritParams draw.evaluated_smooth
 ##'
@@ -60,7 +63,8 @@
 ##'
 ##' ## Alternatively use simulate new data from the model, which
 ##' ## allows construction of reference intervals for the Q-Q plot
-##' qq_plot(m, method = "simulate")
+##' qq_plot(m, method = "simulate", point_col = "steelblue",
+##'         point_alpha = 0.4)
 ##'
 ##' ## ... or use the usual normality assumption
 ##' qq_plot(m, method = "normal")
@@ -71,9 +75,22 @@
                           level = 0.9,
                           ylab = NULL, xlab = NULL,
                           title = NULL, subtitle = NULL, caption = NULL,
-                          alpha = 0.2, ...) {
+                          ci_col = "black",
+                          ci_alpha = 0.2,
+                          point_col = "black",
+                          point_alpha = 1,
+                          line_col = "red", ...) {
     method <- match.arg(method)         # what method for the QQ plot?
-
+    ## check if we can do the method
+    if (identical(method, "direct") &&
+        is.null(fix.family.qf(family(model))[["qf"]])) {
+        method <- "simulate"
+    }
+    if (identical(method, "simulate") &&
+        is.null(fix.family.rd(family(model))[["rd"]])) {
+        method <- "normal"
+    }
+    
     if (level <= 0 || level >= 1) {
         stop("Level must be 0 < level < 1. Supplied level <", level, ">",
              call. = FALSE)
@@ -120,17 +137,18 @@
     plt <- ggplot(df, aes_string(x = "theoretical", y = "residuals"))
 
     ## add reference line
-    plt <- plt + geom_abline(slope = 1, intercept = 0, col = "red")
+    plt <- plt + geom_abline(slope = 1, intercept = 0, col = line_col)
 
     ## add reference interval
     if (identical(method, "simulate")) {
         plt <- plt + geom_ribbon(aes_string(ymin = "lower", ymax = "upper",
                                             x = "theoretical"),
-                                 inherit.aes = FALSE, alpha = alpha)
+                                 inherit.aes = FALSE,
+                                 alpha = ci_alpha, fill = ci_col)
     }
 
     ## add point layer
-    plt <- plt + geom_point()
+    plt <- plt + geom_point(colour = point_col, alpha = point_alpha)
 
     ## add labels
     plt <- plt + labs(title = title, subtitle = subtitle, caption = caption,
@@ -140,6 +158,29 @@
     plt
 }
 
+##' @export
+##' @rdname qq_plot
+`qq_plot.glm` <- function(model, ...) {
+    if (is.null(model[["sig2"]])) {
+        model[["sig2"]] <- summary(model)$dispersion
+    }
+    qq_plot.gam(model, ...)
+}
+
+##' @importFrom stats df.residual
+`qq_plot.lm` <- function(model, ...) {
+    r <- residuals(model)
+    r.df <- df.residual(model)
+    model[["sig2"]] <- sum((r- mean(r))^2) / r.df
+    if (is.null(weights(model))) {
+        model$prior.weights <- rep(1, nrow(model.frame(model)))
+    }
+    if (is.null(model[["linear.predictors"]])) {
+        model[["linear.predictors"]] <- model[["fitted.values"]]
+    }
+    qq_plot.gam(model, ...)
+}
+
 ##' @importFrom mgcv fix.family.rd
 ##' @importFrom stats weights
 `qq_simulate` <- function(model, n = 50, type = c("deviance","response","pearson"),
@@ -156,10 +197,20 @@
     }
 
     dev_resid_fun <- family[["dev.resids"]] # deviance residuals function
+    if (is.null(dev_resid_fun)) {
+        dev_resid_fun <- family$residuals
+        if (is.null(dev_resid_fun)) {
+            stop("Deviance residual function for family <", family[["family"]],
+                 "> not available.")
+        }
+    }
     var_fun <- family[["variance"]]         # variance function
     fit <- fitted(model)
     prior_w <- weights(model, type = "prior")
     sigma2 <- model[["sig2"]]
+    ## if (is.null(sigma2)) {
+    ##     sigma2 <- summary(model)$dispersion
+    ## }
     na_action <- na.action(model)
 
     sims <- replicate(n = n,
@@ -167,7 +218,7 @@
                                        sigma2 = sigma2, dev_resid_fun = dev_resid_fun,
                                        var_fun = var_fun, type = type,
                                        na_action = na_action))
-    n_obs <- length(fit)
+    n_obs <- NROW(fit)
     out <- quantile(sims, probs = (seq_len(n_obs) - 0.5) / n_obs)
     int <- apply(sims, 1L, quantile, probs = c(alpha, 1 - alpha))
     out <- data.frame(theoretical = out, lower = int[1L, ], upper = int[2L, ])
@@ -213,9 +264,9 @@
     fit <- fitted(model)
     weights <- weights(model, type = "prior")
     sigma2 <- model[["sig2"]]
-    if (is.null(sigma2)) {
-        sigma2 <- summary(model)$dispersion # rather than call summary, do only what it does?
-    }
+    ## if (is.null(sigma2)) {
+    ##     sigma2 <- summary(model)$dispersion # rather than call summary, do only what it does?
+    ## }
     na_action <- na.action(model)
     nr <- length(r)                     # number of residuals
     unif <- (seq_len(nr) - 0.5) / nr
@@ -270,16 +321,23 @@
 }
 
 `deviance_residuals` <- function(y, fit, weights, dev_resid_fun) {
-    ## compute deviance residuals
-    r <- dev_resid_fun(y, fit, weights)
-    ## sign of residuals is typically an attribute
-    posneg <- attr(r, "sign")
-    ## ...but may be missing for some families
-    if (is.null(posneg)) {
-        posneg <- sign(y - fit)
+    if ("object" %in% names(formals(dev_resid_fun))) {
+        r <- dev_resid_fun(list(y = y, fitted.values = fit,
+                                prior.weights = weights),
+                           type = "deviance")
+    } else {
+        ## compute deviance residuals
+        r <- dev_resid_fun(y, fit, weights)
+        ## sign of residuals is typically an attribute
+        posneg <- attr(r, "sign")
+        ## ...but may be missing for some families
+        if (is.null(posneg)) {
+            posneg <- sign(y - fit)
+        }
+        ## calculate the deviance residuals
+        r <- sqrt(pmax(r, 0)) * posneg
     }
-    ## colculate the deviance residuals
-    sqrt(pmax(r, 0)) * posneg
+    r
 }
 
 `pearson_residuals` <- function(y, fit, weights, var_fun) {
@@ -306,6 +364,12 @@
 ##'   [ggplot2::labs()].
 ##' @param caption character or expression; the plot caption. See
 ##'   [ggplot2::labs()].
+##' @param point_col colour used to draw points in the plots. See
+##'   [graphics::par()] section **Color Specification**. This is passed to
+##'   the individual plotting functions, and therefore affects the points of
+##'   all plots.
+##' @param point_alpha numeric; alpha transparency for points in plots.
+##' @param line_col colour specification for 1:1 line.
 ##'
 ##' @export
 ##'
@@ -315,7 +379,9 @@
 `residuals_linpred_plot` <- function(model,
                                      type = c("deviance", "pearson","response"),
                                      ylab = NULL, xlab = NULL, title = NULL,
-                                     subtitle = NULL, caption = NULL) {
+                                     subtitle = NULL, caption = NULL,
+                                     point_col = "black", point_alpha = 1,
+                                     line_col = "red") {
     type <- match.arg(type)
     r <- residuals(model, type = type)
     eta <- model[["linear.predictors"]]
@@ -328,10 +394,10 @@
 
     df <- data.frame(eta = eta, residuals = r)
     plt <- ggplot(df, aes_string(x = "eta", y = "residuals")) +
-        geom_hline(yintercept = 0, col = "red")
+        geom_hline(yintercept = 0, col = line_col)
 
     ## add point layer
-    plt <- plt + geom_point()
+    plt <- plt + geom_point(colour = point_col, alpha = point_alpha)
 
     ## add labels
     if (is.null(xlab)) {
@@ -362,9 +428,15 @@
 ##' @importFrom ggplot2 ggplot aes_string geom_point labs
 `observed_fitted_plot` <- function(model,
                                    ylab = NULL, xlab = NULL, title = NULL,
-                                   subtitle = NULL, caption = NULL) {
+                                   subtitle = NULL, caption = NULL,
+                                   point_col = "black", point_alpha = 1) {
     ## extract data for plot
     fit <- fitted(model)
+    ## handle case where fitted is a matrix; extended.families
+    ##   - the needs to be more involved as what about mvn or multinom familities
+    if (NCOL(fit) > 1L) {
+        fit <- fit[, 1]
+    }
     obs <- model[["y"]]
 
     df <- data.frame(observed = obs, fitted = fit)
@@ -373,7 +445,7 @@
     plt <- ggplot(df, aes_string(x = "fitted", y = "observed"))
 
     ## add point layer
-    plt <- plt + geom_point()
+    plt <- plt + geom_point(colour = point_col, alpha = point_alpha)
 
     ## add labels
     if (is.null(xlab)) {
@@ -476,8 +548,15 @@
 ##'   [cowplot::plot_grid()].
 ##' @param level numeric; the coverage level for QQ plot reference intervals.
 ##'   Must be strictly `0 < level < 1`. Only used with `method = "simulate"`.
-##' @param alpha numeric; the level of alpha transparency for the QQ plot
-##'   reference interval when `method = "simulate"`.
+##' @param ci_alpha,ci_col numeric; the level of alpha transparency for the
+##'   QQ plot reference interval when `method = "simulate"`, or points drawn in
+##'   plots.
+##' @param point_col,point_alpha colour and transparency used to draw points in
+##'   the plots. See [graphics::par()] section **Color Specification**. This is
+##'   passed to the individual plotting functions, and therefore affects the
+##'   points of all plots.
+##' @param line_col colour specification for the 1:1 line in the QQ plot and the
+##'   reference line in the residuals vs linear predictor plot.
 ##' @param ... arguments passed to [cowplot::plot_grid()], except for `align`
 ##'   and `axis`, which are set internally.
 ##'
@@ -495,15 +574,23 @@
 ##' ## simulate some data...
 ##' dat <- gamSim(1, n = 400, dist = "normal", scale = 2)
 ##' mod <- gam(y ~ s(x0) + s(x1) + s(x2) + s(x3), data = dat)
-##' ## run some basic model checks, including checking
-##' ## smoothing basis dimensions...
-##' appraise(mod)
-`appraise` <- function(model,
+##' ## run some basic model checks
+##' appraise(mod, point_col = "steelblue", point_alpha = 0.4)
+`appraise` <- function(model, ...) {
+    UseMethod("appraise")
+}
+
+##' @rdname appraise
+##' @export
+`appraise.gam` <- function(model,
                        method = c("direct", "simulate", "normal"),
                        n_uniform = 10, n_simulate = 50,
                        type = c("deviance", "pearson", "response"),
                        n_bins = c("sturges", "scott", "fd"),
-                       ncol = 2, level = 0.9, alpha = 0.2,
+                       ncol = 2, level = 0.9,
+                       ci_col = "black", ci_alpha = 0.2,
+                       point_col = "black", point_alpha = 1,
+                       line_col = "red",
                        ...) {
     ## process args
     method <- match.arg(method)
@@ -518,12 +605,36 @@
     }
 
     plt1 <- qq_plot(model, method = method, type = type, n_uniform = n_uniform,
-                    n_simulate = n_simulate, level = level, alpha = alpha)
-    plt2 <- residuals_linpred_plot(model, type = type)
+                    n_simulate = n_simulate, level = level, ci_alpha = ci_alpha,
+                    point_col = point_col, point_alpha = point_alpha,
+                    line_col = line_col)
+    plt2 <- residuals_linpred_plot(model, type = type, point_col = point_col,
+                                   point_alpha = point_alpha,
+                                   line_col = line_col)
     plt3 <- residuals_hist_plot(model, type = type, n_bins = n_bins,
                                 subtitle = NULL)
-    plt4 <- observed_fitted_plot(model, subtitle = NULL)
+    plt4 <- observed_fitted_plot(model, subtitle = NULL, point_col = point_col,
+                                 point_alpha = point_alpha)
 
     plot_grid(plt1, plt2, plt3, plt4, ncol = ncol, align = "hv",
               axis = "lrtb", ...)
 }
+
+##' @rdname appraise
+##' @importFrom stats model.frame model.response df.residual
+##' @export
+`appraise.lm` <- function(model, ...) {
+    r <- residuals(model)
+    r.df <- df.residual(model)
+    model[["sig2"]] <- sum((r- mean(r))^2) / r.df
+    if (is.null(weights(model))) {
+        model$prior.weights <- rep(1, nrow(model.frame(model)))
+    }
+    if (is.null(model[["linear.predictors"]])) {
+        model[["linear.predictors"]] <- model[["fitted.values"]]
+    }
+    if (is.null(model[["y"]])) {
+        model[["y"]] <- model.response(model.frame(model))
+    }
+    appraise.gam(model, ...)
+}
