@@ -786,3 +786,166 @@
     }
     invisible(res)
 }
+
+## check if a model is a gamm4 object
+`is_gamm4` <- function(object) {
+    out <- FALSE
+    ## is object a list?
+    if (!inherits(object, "list")) {
+        return(out)
+    }
+    nms <- names(object)
+    if (! all(c("gam","mer") %in% nms)) {
+        return(out)
+    }
+    if (! (inherits(object[["mer"]], "lmerMod") &&
+           inherits(object[["gam"]], "gam"))) {
+        return(out)
+    }
+    out <- TRUE
+    out
+}
+
+##' Is a model term a factor (categorical)?
+##'
+##' Given the name (a term label) of a term in a model, identify if the term is a
+##' factor term or numeric. This is useful when considering interactions, where
+##' terms like `fac1:fac2` or `num1:fac1` may be requested by the user. Only for
+##' terms of the type `fac1:fac2` will this function return `TRUE`.
+##'
+##' @param object an R object on which method dispatch is performed
+##' @param term character; the name of a model term, in the sense of
+##'   `attr(terms(object), "term.labels")`. Currently not checked to see if the
+##'   term exists in the model.
+##' @param ... arguments passed to other methods.
+##'
+##' @return A logical: `TRUE` if and only if all variables involved in the term
+##'   are factors, otherwise `FALSE`.
+##'
+##' @export
+`is_factor_term` <- function(object, term, ...) {
+    UseMethod("is_factor_term", object)
+}
+
+##' @rdname is_factor_term
+##' @export
+`is_factor_term.terms` <- function(object, term, ...) {
+    if (missing(term)) {
+        stop("Argument 'term' must be provided.")
+    }
+    facs <- attr(object, "factors")
+    out <- if (term %in% colnames(facs)) {
+        facs <- facs[, term, drop = FALSE]
+        take <- rownames(facs)[as.logical(facs)]
+        data_types <- attr(object, 'dataClasses')[take]
+        all(data_types == "factor")
+    } else {
+        NULL
+    }
+    out
+}
+
+##' @rdname is_factor_term
+##' @export
+`is_factor_term.gam` <- function(object, term, ...) {
+    object <- terms(object)
+    is_factor_term(object, term, ...)
+}
+
+##' @rdname is_factor_term
+##' @export
+`is_factor_term.bam` <- function(object, term, ...) {
+    object <- terms(object)
+    is_factor_term(object, term, ...)
+}
+
+##' @rdname is_factor_term
+##' @export
+`is_factor_term.gamm` <- function(object, term, ...) {
+    object <- terms(object$gam)
+    is_factor_term(object, term, ...)
+}
+
+##' @rdname is_factor_term
+##' @export
+`is_factor_term.list` <- function(object, term, ...) {
+    if (!is_gamm4(object)) {
+        if (all(vapply(object, inherits, logical(1), "terms"))) {
+            out <- any(unlist(lapply(object, is_factor_term, term)))
+        } else {
+            stop("Don't know how to handle generic list objects.")
+        }
+    } else {
+        object <- terms(object$gam)
+        out <- is_factor_term(object, term, ...)
+    }
+    out
+}
+
+##' Names of variables involved in a specified model term
+##'
+##' Given the name (a term label) of a term in a model, returns the names
+##' of the variables involved in ther term.
+##'
+##' @param object an R object on which method dispatch is performed
+##' @param term character; the name of a model term, in the sense of
+##'   `attr(terms(object), "term.labels")`. Currently not checked to see if the
+##'   term exists in the model.
+##' @param ... arguments passed to other methods.
+##'
+##' @return A character vector of variable names.
+##'
+`term_variables` <- function(object, term, ...) {
+    UseMethod("terms_variables", object)
+}
+
+##' @rdname term_variables
+##' @export
+`term_variables.terms` <- function(object, term, ...) {
+    facs <- attr(object, "factors")[ , term]
+    names(facs)[as.logical(facs)]
+}
+
+##' @rdname term_variables
+##' @export
+`term_variables.gam` <- function(object, term, ...) {
+    object <- terms(object)
+    term_variables(object, term, ...)
+}
+
+##' @rdname term_variables
+##' @export
+`term_variables.bam` <- function(object, term, ...) {
+    object <- terms(object)
+    term_variables(object, term, ...)
+}
+
+## Create *mgcv*-alike labels for by smooths
+## FIXME: should make this work for continuous by too
+`mgcv_by_smooth_labels` <- function(smooth, by_var, level) {
+    paste0(smooth, ":", by_var, level)
+}
+
+##' Returns names of variables from a smooth label
+##'
+##' @param label character; a length 1 character vector containing the label of
+##'   a smooth.
+##'
+##' @export
+##' 
+##' @importFrom vctrs vec_c
+##'
+##' @examples
+##'
+##' vars_from_label("s(x1)")
+##' vars_from_label("t2(x1,x2,x3)")
+vars_from_label <- function(label) {
+    if (length(label) > 1) {
+        label <- rep(label, length.out = 1)
+        warning("'label' must be a length 1 vector; using 'label[1]' only.")
+    }
+    vars <- gsub("^[[:alnum:]]{1,2}\\(([[:graph:]]+)\\)$",
+                 "\\1",
+                 label)
+    vec_c(strsplit(vars, ",")[[1L]])
+}
